import datetime
import os
import traceback
import requests
from instabuy_integration_utils.config import config
from instabuy_integration_utils.update import IBSelfUpdate


class IBException:

    @staticmethod
    def save_exception(exception):
        
        try:
            robot_name = config.get("robot_name")
        except AttributeError:
            robot_name = "generic"

        data = {
            'error': str(exception),
            'created_at': datetime.datetime.utcnow(),
            'robot': robot_name,
            'version': IBSelfUpdate.get_local_version(),
            'traceback': traceback.format_exception(etype=type(exception), value=exception, tb=exception.__traceback__)
        }

        response = requests.post(f'{config.api_url}/api_py_robots/errors', data=data)
        if response.status_code != 200:
            with open(os.path.join(config.program_path, 'log_errors.txt'), 'a+', encoding='utf-8') as my_file:
                my_file.write('\n\n\n' + ''.ljust(70, '*') + '\n')
                my_file.write(str(data))
                my_file.write('\n' + ''.ljust(70, '*') + '\n')



from .IBAddress import IBAddress
from .IBBuy import IBBuy
from .IBCheckoutOption import IBCheckoutOption
from .IBClient import IBClient
from .IBCoupon import IBCoupon
from .IBErpIntegration import IBErpIntegration
from .IBOrderProduct import IBOrderProduct



class IBCheckoutOption:
    def __init__(self, checkout_option_dict: dict):
        self.order_item_name = checkout_option_dict["order_item_name"]
        self.additional_price = checkout_option_dict["additional_price"]



class IBAddress:

    def __init__(self, address_dict: dict):
        self.zipcode: str = address_dict['zipcode']
        self.state: str = address_dict['state']
        self.city: str = address_dict['city']
        self.neighborhood: str = address_dict['neighborhood']
        self.street: str = address_dict['street']
        self.street_number: str = address_dict['street_number']
        self.complement: str = address_dict['complement']
        self.country: str = address_dict['country']



import requests
import zipfile
import os
import tempfile


class IBSelfUpdate:

    def __init__(self,
                 program_path: str,
                 retain: list = ["Define.py"],
                 url: str = "http://localhost/",
                 robot_name: str = "robot.zip",
                 version_file_name: str = "version.txt"
                 ) -> None:

        self.retain = retain
        self.url = url
        self.robot_name = robot_name
        self.version_file_name = version_file_name
        self.program_path = os.path.dirname(os.path.abspath(program_path))
        self.__temp_files = {}

    def auto_update(self):
        print("Verifying version")
        if self.verify_version():
            print("Already up to date")
            return

        print("Getting backup files")
        self.retain_files()

        print("Getting new files")
        file_path = self.download_file(self.program_path, self.url + self.robot_name)

        print("Extracting")
        self.extract_file(file_path, self.program_path)

        print("Removing .zip")
        self.rm_file(file_path)

        print("Coping backup files")
        self.backup_files()

        print("Finished")

    def verify_version(self):
        local_version_path = os.path.join(self.program_path, "robot", self.version_file_name)

        if not os.path.exists(local_version_path):
            return False

        local_version = ''
        with open(local_version_path, "r") as local_version_file:
            local_version = local_version_file.read()

        response = requests.get(self.url + self.version_file_name)
        remote_version = response.text

        print("Remote %s --> Local %s" % (remote_version, local_version))

        if local_version != remote_version:
            return False

        return True

    def retain_files(self):
        self.__temp_files = {}

        for file in self.retain:
            file_path = os.path.join(self.program_path, file)

            if not os.path.exists(file_path):
                continue

            temp_file = tempfile.NamedTemporaryFile()
            self.__temp_files[file] = temp_file

            with open(file_path, "rb") as original:
                temp_file.write(original.read())

    def backup_files(self):
        for file in self.__temp_files:
            with open(os.path.join(self.program_path, file), "wb") as backup:
                backup.write(self.__temp_files[file].read())

            self.__temp_files[file].close()

    @staticmethod
    def download_file(path, url):
        local_filename = url.split('/')[-1]

        with requests.get(url, stream=True) as response:
            response.raise_for_status()

            file_path = os.path.join(path, local_filename)
            with open(file_path, 'wb') as file:
                for chunk in response.iter_content(chunk_size=8192):
                    file.write(chunk)
        return file_path

    @staticmethod
    def extract_file(file_path: str, to_dir_path: str):
        if os.path.exists(file_path):
            with zipfile.ZipFile(file_path, 'r') as zip_ref:
                zip_ref.extractall(to_dir_path)
                return True
        return False

    @staticmethod
    def rm_file(file_path):
        if os.path.exists(file_path):
            os.remove(file_path)
            return True
        return False



import time
from typing import Dict, List

import requests
from instabuy_integration_utils.config import config
from instabuy_integration_utils.models.IBBuy import IBBuy
from instabuy_integration_utils.utils.IBProductsLoadUtil import IBProductsLoadUtil


class IBRequest:

    @staticmethod
    def get_buys_ready_for_erp(ib_api_key: str) -> List[IBBuy]:
        response = requests.get(
            "https://api.instabuy.com.br/store/buys?status=erp_ready",
            headers={"api-key": ib_api_key},
        )
        if response.status_code != 200:
            raise Exception("Error on get_buys_ready_for_erp: {response.text}")
        else:
            buys: List[IBBuy] = []
            for buy_dict in response.json()["data"]:
                buys.append(IBBuy(buy_dict))

            return buys

    @staticmethod
    def update_buy_status(
        ib_api_key: str,
        new_status: str,
        request_author: str,
        erp_id=None,
        buy_id=None,
    ):
        body = {"status": new_status, "request_author": request_author}
        if erp_id:
            body["erp_id"] = int(erp_id)
        elif buy_id:
            body["id"] = str(buy_id)
        else:
            raise Exception("You must pass erp_id or buy_id")

        response = requests.put(
            "https://api.instabuy.com.br/store/buys",
            json=body,
            headers={"api-key": ib_api_key},
        )

        if response.status_code != 200:
            raise Exception(f"Error on update_buy_status: {response.text}")

    @staticmethod
    def send_products_to_api(ib_api_key: str, products: List[Dict]):
        if config.DEBUG:
            print("Items processed. Will send to API")

        outdated_products = IBProductsLoadUtil.get_outdated_products(
            ib_api_key, products
        )
        outdated_len = len(outdated_products)

        if outdated_len == 0:
            if config.DEBUG:
                print("There is none outdated product to send")

            return

        if config.DEBUG:
            print(
                f"Will send {outdated_len} outdated products from {len(products)} products"
            )

        products_count = 0
        while len(outdated_products) != 0:
            products_to_send = []

            for _ in range(
                0, min(config.items_batch_count, len(outdated_products)), +1
            ):
                products_to_send.append(outdated_products[0])
                outdated_products.pop(0)

            products_count += len(products_to_send)

            if config.DEBUG:
                init_time = time.time()
                print(f"Sending {products_count}/{outdated_len} products...")

            response = requests.put(
                "https://api.instabuy.com.br/store/products",
                json={"products": products_to_send},
                headers={"api-key": ib_api_key},
            )

            if response.status_code != 200:
                raise Exception(f"Error on send_products_to_api: {response.text}")

            if config.DEBUG:
                print(f"Request processed in {int(time.time() - init_time)} seconds")

        if config.DEBUG:
            print("Items updated")



import hashlib
import json
import os
from datetime import datetime
from typing import Dict, List
from instabuy_integration_utils.config import config


class IBProductsLoadUtil:
    @staticmethod
    def get_outdated_products(ib_api_key: str, products: List[Dict]) -> List[Dict]:
        today = datetime.utcnow().day
        actual_load_hashes = {}
        outdated_products = []
        last_load_products_log = IBProductsLoadUtil.__get_last_load(ib_api_key)

        for product in products:
            internal_code = product["internal_code"]
            product_hash = hashlib.sha256(str(product).encode()).hexdigest()

            actual_load_hashes[internal_code] = {
                "day": today,
                "hash": product_hash,
            }

            if (
                internal_code not in last_load_products_log
                or last_load_products_log[internal_code]["hash"] != product_hash
                or last_load_products_log[internal_code]["day"] != today
            ):
                outdated_products.append(product)

        IBProductsLoadUtil.__save_load(ib_api_key, actual_load_hashes)
        return outdated_products

    @staticmethod
    def __get_last_load(ib_api_key: str) -> Dict:
        cache_file_path = IBProductsLoadUtil.__get_file_path(ib_api_key)
        if os.path.isfile(cache_file_path):
            with open(cache_file_path, "r") as products_file:
                products_hash = json.load(products_file)["products"]
                return products_hash

        return {}

    @staticmethod
    def __save_load(ib_api_key: str, products_hash_update: Dict):
        cache_file_path = IBProductsLoadUtil.__get_file_path(ib_api_key)
        with open(cache_file_path, "w") as products_load_file:
            json.dump(
                {
                    "started_at": datetime.utcnow().isoformat(),
                    "products": products_hash_update,
                },
                products_load_file,
            )

    @staticmethod
    def __get_file_path(
        ib_api_key: str,
    ) -> str:
        if not os.path.exists(config.ib_integration_temp_files):
            os.makedirs(config.ib_integration_temp_files, exist_ok=True)

        return os.path.join(
            config.ib_integration_temp_files,
            f"ib{hashlib.sha256(ib_api_key.encode()).hexdigest()}.json",
        )



import json
import os
import sys


class __Config:

    def __init__(self):
        self.api_url = 'https://api.instabuy.com.br'
        self.items_batch_count = 1000
        self.program_path: str = os.path.dirname(os.path.abspath(sys.argv[0]))
        self.ib_integration_temp_files = os.path.join(self.program_path, 'tmp')
        self.version_file_name: str = "version.txt"
        self.DEBUG = False

    def get_local_version(self):
        local_vpath = os.path.join(self.program_path, self.version_file_name)

        if not os.path.exists(local_vpath):
            return None

        local_version = ''
        with open(local_vpath, "r", encoding='utf-8') as local_vfile:
            local_version = local_vfile.read()

        return local_version

    def load_local_config(self, local_file_name="config.json"):
        with open(os.path.join(self.program_path, local_file_name), 'r') as local_file:
            local_config = (json.load(local_file))

            for key in local_config:
                self.__setattr__(key, local_config[key])


config = __Config()



from abc import abstractmethod


class IBErpIntegration:

    @staticmethod
    @abstractmethod
    def update_prices_and_stocks():
        pass

    @staticmethod
    @abstractmethod
    def sync_buys():
        pass



import datetime
from typing import List
from .IBAddress import IBAddress
from .IBCheckoutOption import IBCheckoutOption
from .IBClient import IBClient
from .IBCoupon import IBCoupon
from .IBOrderProduct import IBOrderProduct
from instabuy_integration_utils.utils.IBUtil import IBUtil


class IBBuy:

    def __init__(self, buy_dict: dict):
        self.id: str = buy_dict["id"]
        self.created_at: datetime = IBUtil.date_from_iso_string(buy_dict["created_at"])
        self.subtotal: float = buy_dict["_subtotal"]
        self.freight: float = buy_dict["freight"]
        self.discount: float = buy_dict["_discount"]
        self.total: float = buy_dict["_total"]
        self.comment: str = buy_dict["comment"]
        self.code: str = buy_dict["code"]
        self.erp_id: int = buy_dict.get("erp_id")
        self.buy_type: str = buy_dict["buy_type"]
        self.was_paid: bool = buy_dict["_was_paid"]
        self._already_injected_delivery_tax_on_products_array = False

        self.client = IBClient(buy_dict["client"])

        if buy_dict.get("delivery_info"):
            self.delivery_info = IBAddress(buy_dict["delivery_info"])
        else:
            self.delivery_info = None

        if buy_dict.get("coupon"):
            self.coupon = IBCoupon(buy_dict["coupon"])
        else:
            self.coupon = None

        if buy_dict.get("checkout_option"):
            self.checkout_option = IBCheckoutOption(buy_dict["checkout_option"])
        else:
            self.checkout_option = None

        self.products: List[IBOrderProduct] = []
        # todo implement kits
        for prod_dict in buy_dict["products"]:
            if prod_dict["qtd"] > 0:
                self.products.append(IBOrderProduct(prod_dict))

        self.payment_method: str = buy_dict["payment_info"]["method"]
        self.payment_value: str = buy_dict["payment_info"].get("value")
        self.installments: int = (
            buy_dict["installment"]["installments_number"]
            if buy_dict.get("installment")
            else 1
        )

        if buy_dict.get("delivery_hour"):
            self.delivery_hour = buy_dict["delivery_hour"]
        else:
            self.delivery_hour = None

    def inject_delivery_tax_on_products_array(self, delivery_tax_internal_code):
        if not self._already_injected_delivery_tax_on_products_array:
            if self.freight > 0:
                delivery_tax_product = IBOrderProduct()
                delivery_tax_product.price = self.freight
                delivery_tax_product.qtd = 1.0
                delivery_tax_product.model_internal_code = delivery_tax_internal_code
                delivery_tax_product.name = "Taxa de Entrega"
                self.products.append(delivery_tax_product)
                self._already_injected_delivery_tax_on_products_array = True

            elif self.checkout_option and self.checkout_option.additional_price:
                checkout_option_product = IBOrderProduct()
                checkout_option_product.price = self.checkout_option.additional_price
                checkout_option_product.qtd = 1.0
                checkout_option_product.model_internal_code = delivery_tax_internal_code
                checkout_option_product.name = self.checkout_option.order_item_name
                self.products.append(checkout_option_product)



import datetime
from instabuy_integration_utils.utils.IBUtil import IBUtil


class IBClient:
    def __init__(self, client_dict: dict):
        self.user_type: str = client_dict["user_type"]
        self.email: str = (
            client_dict["email"] if client_dict.get("email") else "sem@email.com"
        )
        self.phone: str = client_dict["phone"]
        self.id: str = client_dict["id"]
        self.created_at: datetime = IBUtil.date_from_iso_string(
            client_dict["created_at"]
        )

        if self.is_pf():
            self.first_name: str = client_dict["first_name"]
            self.last_name: str = client_dict["last_name"]
            self.cpf: str = client_dict["cpf"]
            self.birthday: datetime = IBUtil.date_from_iso_string(
                client_dict["birthday"]
            )
            self.gender: str = client_dict["gender"]
        else:
            self.company_name: str = client_dict["company_name"]
            self.fantasy_name: str = client_dict["fantasy_name"]
            self.cnpj: str = client_dict["cnpj"]

    def is_pf(self) -> bool:
        return self.user_type == "PF"

    def get_document(self) -> str:
        return IBUtil.get_only_numbers_from_string(
            self.cpf if self.is_pf() else self.cnpj
        )

    def get_complete_name(self) -> str:
        if self.is_pf():
            return f"{self.first_name} {self.last_name}"

        return self.fantasy_name



from instabuy_integration_utils import utils
from instabuy_integration_utils import models
from instabuy_integration_utils.update import IBSelfUpdate
from instabuy_integration_utils.config import config



import datetime
import os
import re
import time
import unicodedata
from dateutil import tz
from unidecode import unidecode


class IBUtil:

    @staticmethod
    def remove_special_characters(content: str) -> str:
        content = unidecode(content)
        string = re.sub(r'[^A-Za-z0-9]+', ' ', content)
        string = "".join(
            (
                c
                for c in unicodedata.normalize("NFD", string)
                if unicodedata.category(c) != "Mn"
            )
        )
        string = string.encode("latin-1", "ignore").decode("latin-1")
        return string

    @staticmethod
    def force_greater_or_equal_0(number) -> float:
        if number is None:
            number = 0.0
        if isinstance(number, str):
            number = number.replace(',', '.')

        try:
            number = float(number)
        except Exception:
            number = 0.0

        return max(number, 0.0)

    @staticmethod
    def get_payment_id(payment_method: str, erp_payments_option: dict) -> str:
        for erp_payments_option in erp_payments_option:
            if payment_method == erp_payments_option['ib_method'] or (
                    '_credit' in payment_method and '_credit' in erp_payments_option['ib_method']):
                return erp_payments_option['erp_id']

        raise Exception('Payment method ID not found.')

    @staticmethod
    def get_client_phone_number(phone: str, ddd=False) -> str:
        if ddd:
            return phone.split(')')[0].replace('(', '')
        else:
            return IBUtil.get_only_numbers_from_string(phone.split(')')[1])

    @staticmethod
    def format_value(value: float, decimal=2) -> str:
        if decimal == 2:
            value = '{:.2f}'.format(value)
        elif decimal == 4:
            value = '{:.4f}'.format(value)
        return ''.join('{:0>13}'.format(str(value)))

    @staticmethod
    def complete_empty_positions(string, size, append_size='right', character=' ') -> str:
        if not string:
            string = ''
        else:
            string = str(string)

        string = string[:size]

        if append_size == 'left':
            return character * (size - len(string)) + string
        else:
            return string + character * (size - len(string))

    @staticmethod
    def date_from_iso_string(date_string: str, force_end_of_day=False) -> datetime:
        if 'Z' in date_string:
            date_string = date_string.replace('Z', '+00:00')
        if len(date_string) == 24:
            date_string = date_string[:-2] + ':' + date_string[-2:]

        date = datetime.datetime.fromisoformat(date_string)
        if not date.tzinfo:
            date = date.astimezone(tz=tz.tzrange(-3))

        if len(date_string) == 10 and force_end_of_day:
            date = date + datetime.timedelta(days=1, seconds=-1)

        date = date.replace(tzinfo=None)

        return date

    @staticmethod
    def check_if_file_is_being_uploaded(file_path):
        while True:
            file_size = os.path.getsize(file_path)
            time.sleep(10)
            if os.path.getsize(file_path) == file_size:
                return

    @staticmethod
    def pop_keys(dic, keys):
        for key in keys:
            if key in dic:
                dic.pop(key)

    @staticmethod
    def string_to_datetime(date_string, force_end_of_day=False) -> datetime:
        date = datetime.datetime.strptime(date_string, '%d/%m/%Y')
        if force_end_of_day:
            date = date.replace(hour=23, minute=59, second=59)
        else:
            date = date.replace(hour=0, minute=0, second=0)

        return date

    @staticmethod
    def get_only_numbers_from_string(string) -> str:
        if not string:
            string = ''
        else:
            string = str(string)

        return ''.join([n for n in string if n.isdigit()])

    @staticmethod
    def remove_folder_or_file(path):
        try:
            os.remove(path)
        except Exception:
            pass

        try:
            os.rmdir(path)
        except Exception:
            pass



from typing import List


class IBOrderProduct:

    def __init__(self, product_dict: dict = None):
        if product_dict:
            self.id: str = product_dict['id']
            self.model_internal_code: str = product_dict['model_internal_code']
            self.name: str = product_dict['name']
            self.price: float = product_dict['price']
            self.qtd: float = product_dict['qtd']
            self.unit_type: str = product_dict['unit_type']
            self.bar_codes: List[str] = product_dict['bar_codes']
